import { useResource } from "@/modules/resource";
import { useQueditor } from "@queenjs-modules/queditor";
import { switchSceneProdComp } from "@queenjs-modules/queditor/module/controls/Queen3dCtrl/actions/geom";
import { Pack, PackMat } from "@queenjs-modules/queditor/objects";
import { IQueentree } from "@queenjs-modules/queentree";
import { AssetItemFile } from "@queenjs-modules/queentree-explorer/objects/fileSystem/assetFiles";
import { List } from "@queenjs/ui";
import { defineComponent } from "vue";
import LibraryModal from "../LibraryModal";
import MatItem from "./MatItem";
import MeshItem from "./MeshItem";

export default defineComponent({
  setup() {
    const queditor = useQueditor();
    const resource = useResource();
    const { store } = resource;

    const replaceMesh = async () => {
      const branchFolder = await resource.showModal<AssetItemFile>(
        <LibraryModal />,
        { width: "900px" }
      );

      const pack = await branchFolder.getAssetDetail();
      console.log("pack: ", pack);
      const data = queditor.helper.getRelatedSourceByProduct(
        pack.source as Pack["source"],
        branchFolder.state.id
      );

      console.error("data: ", data);
      queditor.actions.insertMesh(data);
    };

    const replaceMat = async (record: any) => {
      const branchFolder = await resource.showModal<AssetItemFile>(
        <LibraryModal nodeTypes={["matGroupItem", "mat", "packMat"]} />,
        { width: "900px" }
      );
      // console.error("branchFolder: ", branchFolder);
      const data = await branchFolder.getAssetDetail();
      // console.error("data: ", data);

      let mat;
      switch (branchFolder.nodeType) {
        case "matGroupItem":
          mat = (data as IQueentree.IAssetMatGroup).source.colorCards.find(
            (d: PackMat) => d.id == branchFolder.state.id
          );
          break;
        case "packMat":
          mat = (data.source as Pack["source"]).mats.find(
            (d: PackMat) => d.id == branchFolder.state.id
          );
          break;
        case "mat":
          mat = data.source as PackMat;
          break;
      }

      if (!data || !mat) return;
      const matConf = resource.store.matSlots.find(
        (item) => item.id == record.id
      );
      if (matConf) {
        matConf.material = mat;
      } else {
        const matSlot = { id: record.id, material: mat };
        resource.store.matSlots.push(matSlot);
      }

      record.thumbnail = mat?.thumbnail?.url;

      // console.error("resource.store.matSlots: ", resource.store.matSlots);

      const targetComp = getTargetProCom(record);
      if (!targetComp || !mat) return;
      queditor.actions.updatePackProductCompMat(targetComp, mat);
      switchComp(record.meshSlotName);

      // const targetMesh = store.treeData.find(
      //   (m: any) => m.Id == record.meshSlotId
      // );
      // store.activeKey = { type: "comp", CompId: targetMesh.Id };
    };

    const getTargetProCom = (record: any) => {
      const source: Pack["source"] = queditor.store.pack;
      const targetPro = source.products[0];
      const targetComp = targetPro?.components.find(
        (c) => c.name == record.meshSlotName
      );
      return targetComp;
    };

    const switchComp = (compName: string) => {
      switchSceneProdComp.call(
        queditor.controls.queen3dCtrl,
        queditor.store.pack.scenes[0].products[0].id,
        compName
      );
    };

    const clickMesh = (item: any) => {
      resource.store.setActiveKey({
        type: "mesh",
        id: item.Id,
      });
      switchComp(item.meshName);
    };

    const clickMat = (item: any) => {
      switchComp(item.meshSlotName);
      resource.store.setActiveKey({
        type: "mat",
        id: item.id,
      });
    };

    return () => {
      const Meshlist = store.treeData || [];

      return (
        <div class="w-300px flex flex-col">
          <div class="p-15px text-16px text-white border-dark-800 border-0 border-b-1 border-solid">
            模型列表
          </div>
          <List
            data={Meshlist}
            gap="10px"
            class="scrollbar flex-1 py-15px px-15px"
          >
            {{
              item: (record: any) => (
                <div>
                  <MeshItem
                    record={record}
                    active={record.Id == store.activeKeys.id}
                    onReplace={replaceMesh}
                    onClick={clickMesh}
                  />
                  <div class="grid gap-y-10px pt-10px pl-10px">
                    {record.children.map((item: any) => (
                      <MatItem
                        key={item.id}
                        record={item}
                        active={item.id == store.activeKeys.id}
                        onClick={clickMat}
                        onReplace={replaceMat}
                      />
                    ))}
                  </div>
                </div>
              ),
              loadmore: () => (
                <div class="text-center py-20px text-12px opacity-80">
                  没有更多了
                </div>
              ),
            }}
          </List>
        </div>
      );
    };
  },
});
